/*
 * USART0.c
 *
 * Created: 7/4/2011 21:34:43
 *  Author: MMM
 */
#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/io.h>
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"
#include "USART0.h"

const portTickType xBlockTime = 0x0000;
//receive and transmit queues
xQueueHandle xRxedChars=NULL; 
xQueueHandle xCharsForTx=NULL; 

void prvUDRIE0InterruptOn(void)
{
 UCSR0B |= (1<<UDRIE0);
}								

void prvUDRIE0InterruptOff(void)
{
UCSR0B &= ~(1<<UDRIE0); 		
}
void vUSART0Init(unsigned portBASE_TYPE uxQueueLength)
{
	// Create the transmit and receive queues
	xRxedChars = xQueueCreate( uxQueueLength, ( signed char ) sizeof( signed char ) );
	xCharsForTx = xQueueCreate( uxQueueLength, ( signed char ) sizeof( signed char ) );
	portENTER_CRITICAL();
    // Set baud rate
    UBRR0H = (uint8_t)(USART0_UBBR_VALUE>>8);
    UBRR0L = (uint8_t)USART0_UBBR_VALUE;

    // Set frame format to 8 data bits, no parity, 1 stop bit
    UCSR0C |= (0<<USBS0)|(1<<UCSZ01)|(1<<UCSZ00);

    /* Enable receiver and transmitter,
	 enable RX complete interrupt*/
    UCSR0B |= (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0);   
	portEXIT_CRITICAL();	
}
portBASE_TYPE xUSART0PutChar(unsigned char cOutChar)
{
//Return false if after the block time there is no room on the Tx queue.
	if( xQueueSend( xCharsForTx, &cOutChar, xBlockTime ) != pdPASS )
	{
		return pdFAIL;
	}
	//enable usart UDRE interrupt to transmit
	prvUDRIE0InterruptOn();
	return pdPASS;
}
portBASE_TYPE xUSART0GetChar(unsigned char *pcRxedChar)
{
/* Get the next character from the buffer.  Return false if no characters
	are available, or arrive before xBlockTime expires. */
	if( xQueueReceive( xRxedChars, pcRxedChar, xBlockTime ) )
	{
		return pdTRUE;
	}
	else
	{
		return pdFALSE;
	}
	
}	
ISR( USART0_RX_vect )
{
signed char cChar;
signed portBASE_TYPE xHigherPriorityTaskWoken;

	/* Get the character and post it on the queue of Rxed characters.
	If the post causes a task to wake force a context switch as the woken task
	may have a higher priority than the task we have interrupted. */
	cChar = UDR0;

	xQueueSendFromISR( xRxedChars, &cChar, &xHigherPriorityTaskWoken );

}
ISR( USART0_UDRE_vect )
{
signed char cChar, cTaskWoken;

	if( xQueueReceiveFromISR( xCharsForTx, &cChar, &cTaskWoken ) == pdTRUE )
	{
		/* Send the next character queued for Tx. */
		UDR0 = cChar;
	}
	else
	{
		/* Queue empty, nothing to send. */
		prvUDRIE0InterruptOff();
	}
	
}
portBASE_TYPE xUSART0SendDataP(const unsigned char *data)
{
	unsigned char i;

	// check to make sure we have a good pointer
	if (!data) return pdFALSE;
	// print data

	for(i=0;(uint8_t)pgm_read_byte(&data[i]);i++)
	{
		xUSART0PutChar((unsigned char)pgm_read_byte(&data[i]));
	}
	return pdTRUE;
}
portBASE_TYPE xUSART0SendData(const unsigned char *data)
{
	unsigned char i;

	// check to make sure we have a good pointer
	if (!data) return pdFALSE;
	// print data

	for(i=0;data[i];i++)
	{
		xUSART0PutChar(data[i]);
	}
	return pdTRUE;
}
